//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011"
#property link      ""

#include <CustPrevCalculated.mqh>

//+------------------------------------------------------------------+
//|                                  |
//+------------------------------------------------------------------+
enum CPCATRResultCode
  {
   CPCATRRC_NotInitialized,             //   
   CPCATRRC_NoData,                     //  
   CPCATRRC_NotChanged,                 //   
   CPCATRRC_Changed                     //  
  };

//+------------------------------------------------------------------+
//|                                     |
//+------------------------------------------------------------------+
struct ATRBar
  {
   double ATR;
   double TR;
  };

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
class CCustATR
  {
protected:
   bool Initialized;                         //         
   bool UsesExternalData;                    //   
   //  
   //----------------------------
   int BarsLimit;                            //    ,     
   uchar iAtrPeriod;
   //    
   //----------------------------
   uint DataBarsCount;                       //        ,     BarsLimit
   uint DataBarsCalculated;                  //     
   uint DataStartInd;                        //        (  )
   ATRBar Buf[];                           //     
   uint ATRDataBarsCalculated;             //    
   //----------------------------
   bool ATRDataBarsCuttingLeft;            //         PrepareData()
   CCustPrevCalculated *CustPrevCalculated;
   //  
   //----------------------------
   int __Digits;
   double __Point;
   uint PInd(uint I, bool AsSeries) {return( AsSeries?((DataStartInd+DataBarsCount-1-I) % BarsLimit):((DataStartInd+I) % BarsLimit) );}
public:
   //   
   void CCustATR() {Initialized = false;}
   void ~CCustATR() {if (CustPrevCalculated != NULL) delete CustPrevCalculated;}
   bool ItIsInitialized() {return(Initialized);}
   //   
   bool Init(CCustPrevCalculated *CPC, string Instr, ENUM_TIMEFRAMES TF, int Limit, CPCHistorySynchMode HSM, uchar HS, uint HSMinute, uchar AtrPeriod);
   //    
   CPCPrepareDataResultCode PrepareData(CPCPrepareDataResultCode resData);   //       
   //    
   CPCATRResultCode Calculate();
   //     
   uint GetBarsCount() {return(DataBarsCount);}                   //    
   uint GetBarsCalculated() {return(ATRDataBarsCalculated);}      //    
   double GetATR(uint shift, bool AsSeries);                      //    ATR  
  };
//+------------------------------------------------------------------+
bool CCustATR::Init(CCustPrevCalculated *CPC, string Instr, ENUM_TIMEFRAMES TF, int Limit, CPCHistorySynchMode HSM, uchar HS, uint HSMinute, uchar AtrPeriod)
{
   if (Initialized)
      return(false);
   
   if (CPC == NULL)
     {
      CustPrevCalculated = new CCustPrevCalculated;
      UsesExternalData = false;
     } else {
      CustPrevCalculated = CPC;
      UsesExternalData = true;
     }
   if (CustPrevCalculated.InitData(Instr, TF, Limit, HSM, HS, HSMinute))
     {
      if (ArrayResize(Buf, Limit) == -1)
         return(Initialized);

      BarsLimit = Limit;
      iAtrPeriod = AtrPeriod;
      
      __Digits = (int)SymbolInfoInteger(Instr, SYMBOL_DIGITS);
      __Point = SymbolInfoDouble(Instr, SYMBOL_POINT);
      
      Initialized = true;
      return(true);
     } else {
      Initialized = false;
      return(false);
     }
}
//+------------------------------------------------------------------+
CPCPrepareDataResultCode CCustATR::PrepareData(CPCPrepareDataResultCode resData)
{
   if (!Initialized)
      return(CPCPDRC_NoData);
   
   //     
   CPCPrepareDataResultCode res = resData;
   DataBarsCount = CustPrevCalculated.GetDataBarsCount();
   DataBarsCalculated = CustPrevCalculated.GetDataBarsCalculated();
   DataStartInd = CustPrevCalculated.GetDataStartInd();
   
   //     ,      
   //   ,     
   if (CustPrevCalculated.GetDataBarsCuttingLeft())
      ATRDataBarsCuttingLeft = true;
   
   //        
   for (uint i = (DataBarsCalculated == 0)?0:(DataBarsCalculated+1); i < DataBarsCount; i++)
     {
      Buf[PInd(i, false)].ATR = EMPTY_VALUE;
      Buf[PInd(i, false)].TR = EMPTY_VALUE;
     }
   
   //     ,    
   ATRDataBarsCalculated = MathMin(DataBarsCalculated, ATRDataBarsCalculated);
   
   return(res);
}
//+------------------------------------------------------------------+
double CCustATR::GetATR(uint shift, bool AsSeries)
{
   if ( shift > (DataBarsCount-1) )
      return(EMPTY_VALUE);
   return(Buf[PInd(shift, AsSeries)].ATR);
}
//+------------------------------------------------------------------+
CPCATRResultCode CCustATR::Calculate()
{
   if (!Initialized)
      return(CPCATRRC_NotInitialized);
   
   //     ,   
   if (!UsesExternalData)
     {
      CPCPrepareDataResultCode resCPCPD = CustPrevCalculated.PrepareData();
      CPCPrepareDataResultCode resPD = PrepareData(resCPCPD);

      if (resPD == CPCPDRC_NoData)
         return(CPCATRRC_NoData);
      if (resPD == CPCPDRC_NoRecountNotRequired)
         return(CPCATRRC_NotChanged);
     }
   
   //  -      
   if (DataBarsCount <= iAtrPeriod)
      return(CPCATRRC_NoData);
   
   uint BarsForRecalculation;
   if ( DataBarsCalculated != 0 )
      BarsForRecalculation = DataBarsCount - ATRDataBarsCalculated - 1;
   else
     {
      Buf[PInd(0, false)].TR = 0.0;
      Buf[PInd(0, false)].ATR = 0.0;
      //--- filling out the array of True Range values for each period
      for (uint i = 1; i < DataBarsCount; i++)
         Buf[PInd(i, false)].TR = MathMax(CustPrevCalculated.GetDataHigh(i, false), CustPrevCalculated.GetDataClose(i-1, false)) - 
                                  MathMin(CustPrevCalculated.GetDataLow(i, false), CustPrevCalculated.GetDataClose(i-1, false));
      //--- first AtrPeriod values of the indicator are not calculated
      double firstValue = 0.0;
      for (uint i = 1; i <= iAtrPeriod; i++)
        {
         Buf[PInd(i, false)].ATR = 0;
         firstValue += Buf[PInd(i, false)].TR;
        }
      //--- calculating the first value of the indicator
      firstValue /= iAtrPeriod;
      Buf[PInd(iAtrPeriod, false)].ATR = firstValue;
      
      BarsForRecalculation = DataBarsCount - iAtrPeriod - 2;
     }

   //    
   if (ATRDataBarsCuttingLeft)
     {
      ATRDataBarsCuttingLeft = false;
     }

   //======================================================
   //========   ===============================
   //======================================================
   CPCATRResultCode res = CPCATRRC_NotChanged;
   
   for (uint i = (DataBarsCount - BarsForRecalculation - 1); i < DataBarsCount; i++)
     {
      Buf[PInd(i, false)].TR = MathMax(CustPrevCalculated.GetDataHigh(i, false), CustPrevCalculated.GetDataClose(i-1, false)) - 
                               MathMin(CustPrevCalculated.GetDataLow(i, false), CustPrevCalculated.GetDataClose(i-1, false));
      Buf[PInd(i, false)].ATR = Buf[PInd(i-1, false)].ATR + (Buf[PInd(i, false)].TR-Buf[PInd(i-iAtrPeriod, false)].TR) / iAtrPeriod;
     
      res = CPCATRRC_Changed;
      ATRDataBarsCalculated = i + 1; //  ,     
     } // for ()
   //----------
   return(res);
}